#!/usr/bin/python
# -*- coding: iso8859-1 -*-

#
# scam2memu
# utility to convert softcam file into intel-hex file for multiemu cam
# by jeanjohn
#
# ver. 0.1.0 21st August 2004
#

DEBUG = False
APPNAME = "scam2memu"
APPVER = "0.1.0"

import os
import re
import sys

class SoftCam:
    def __init__(self, ca, provId, keyNum, key, key2 = None):
        self.ca = ca # conditioned access type
        self.provId = provId
        self.keyNum = keyNum
        self.key = key
        self.key2 = key2 # only for conax

    def __str__(self):
        if self.key2:
            return " ".join([self.ca, self.provId, self.keyNum, self.key, self.key2])
        else:            
            return " ".join([self.ca, self.provId, self.keyNum, self.key])

SECA2_PROVID_START = 0x0064
SOFTCAM_RE = re.compile("^([SX])\s+([0-9A-F]+)\s+([0-9A-F]+)\s+([0-9A-F]+)\s+([0-9A-F]+)?.*$")

def readSoftCamLine(line):
    m = SOFTCAM_RE.match(line)
    if m and not (m.group(1) == "S" and int(m.group(2),16) < SECA2_PROVID_START):       
        return SoftCam(m.group(1), m.group(2), m.group(3), m.group(4), m.group(5))
    return None

def formatEmuFile(scList):
    s = ""
    for sc in scList:
        if sc.ca == "X":
            s += "".join(["01", sc.keyNum, sc.key, sc.key2])
        elif sc.ca == "S":
            s += "".join(["02", sc.provId, sc.keyNum, sc.key])
    return s

def writeIntelHex(fd, emuFile):
    # Intel Hex format:
    # ":" startcode
    # "bb" bytecount
    # "aaaa" load address
    # "tt" type
    # "dd...dd" data
    # "cc" checksum
    filelen = len(emuFile) / 2 # 2 ascii-hex per byte
    pos = 0;
    while pos < filelen:
        reclen = 16
        if (filelen - pos) < reclen:
            reclen = filelen - pos
        fd.write(":")
        head = "%02X%04X00" % (reclen, pos)
        cksum = 0
        toWrite = head + emuFile[pos*2:(pos+reclen)*2+1]
        for i in xrange(len(toWrite)/2):
            data = toWrite[i*2:i*2+2]
            fd.write(data)
            cksum = (cksum + int(data, 16)) & 0xff
        cksum = (0x0100 - cksum) & 0xff # 2-complement of sum
        fd.write("%02X\n" % cksum)
        pos += reclen
    fd.write(":00000001FF\n")
    fd.close()
    
def main():
    # banner
    print >>sys.stderr, "%s ver. %s" % (APPNAME, APPVER)
    # open softcam file
    fd = None
    if len(sys.argv) >= 2:
        fd = open(sys.argv[1], "r")
    else:
        print >>sys.stderr, "Usage: %s SoftCam.key" % APPNAME
        print >>sys.stderr, "Error: softcam key file not specified!"
        return -1

    # read softcam file
    scList = []
    line = fd.readline()
    while line:
        sc = readSoftCamLine(line)
        if (sc):
            scList.append(sc)
            if DEBUG:
                print sc
        line = fd.readline()
    # close softcam file        
    fd.close()
    if len(scList) == 0:
        print >>sys.stderr, "Unable to find Seca2 or Conax keys!"
        return -2
    # generate intel-hex file
    emufile = formatEmuFile(scList)
    emufile = "%04X" % (len(emufile)/2) + emufile
    if DEBUG:
        print emufile
    filename = ".".join([os.path.splitext(sys.argv[1])[0], "hex"])
    fd = open(filename, "w")
    writeIntelHex(fd, emufile)
    fd.close()
    return 0

if __name__ == "__main__":
    main()
